import os
import imageio
from collections import defaultdict
import shutil

def get_all_mp4_files(source_main_folder):
    """
    遍历主文件夹中的所有子文件夹，获取所有mp4文件的路径。
    返回一个字典，键为文件名，值为所有对应路径的列表。
    """
    video_dict = defaultdict(list)
    for root, dirs, files in os.walk(source_main_folder):
        for file in files:
            if file.lower().endswith('.mp4'):
                file_path = os.path.join(root, file)
                video_dict[file].append(file_path)
    return video_dict

def get_frame_count(video_path):
    """
    获取视频的总帧数。
    """
    try:
        reader = imageio.get_reader(video_path, 'ffmpeg')
        count = reader.count_frames()
        reader.close()
        return count
    except Exception as e:
        print(f"无法读取视频 {video_path} 的帧数: {e}")
        return None

def process_and_save_video(source_path, target_path, frame_count, desired_fps):
    """
    读取源视频的前frame_count帧，保存到目标路径，设置为desired_fps。
    """
    try:
        reader = imageio.get_reader(source_path, 'ffmpeg')
        writer = imageio.get_writer(target_path, fps=desired_fps, codec='libx264')

        for i, frame in enumerate(reader):
            if i >= frame_count:
                break
            writer.append_data(frame)

        reader.close()
        writer.close()
        print(f"已处理并保存视频: {target_path}")
    except Exception as e:
        print(f"处理视频 {source_path} 时出错: {e}")

def ensure_dir_exists(path):
    """
    确保目标文件夹存在。
    """
    os.makedirs(path, exist_ok=True)

def main():
    # 定义源主文件夹和目标主文件夹路径
    source_main_folder = '/path/to/source_main_folder'   # 替换为您的源主文件夹路径
    target_main_folder = '/path/to/target_main_folder'   # 替换为您的目标主文件夹路径

    # 定义新的帧率
    desired_fps = 30  # 替换为您需要的帧率值，如24, 30, 60等

    # 检查源文件夹是否存在
    if not os.path.exists(source_main_folder):
        print(f"源文件夹不存在: {source_main_folder}")
        return

    # 如果目标文件夹已存在，可以选择是否清空
    if os.path.exists(target_main_folder):
        response = input(f"目标文件夹 '{target_main_folder}' 已存在。是否删除并继续？ (y/n): ")
        if response.lower() == 'y':
            shutil.rmtree(target_main_folder)
            ensure_dir_exists(target_main_folder)
        else:
            print("操作已取消。")
            return
    else:
        ensure_dir_exists(target_main_folder)

    # 获取所有mp4文件
    video_dict = get_all_mp4_files(source_main_folder)

    for filename, paths in video_dict.items():
        if len(paths) < 2:
            print(f"警告: 文件 {filename} 只在 {len(paths)} 个文件夹中找到，可能不足以比较帧数。")
            # 这里根据需求决定是否跳过
            # continue

        # 获取每个视频的帧数
        frame_counts = []
        for path in paths:
            fc = get_frame_count(path)
            if fc is not None:
                frame_counts.append(fc)

        if not frame_counts:
            print(f"无法获取任何帧数信息，跳过文件 {filename}.")
            continue

        min_frame_count = min(frame_counts)
        print(f"文件 {filename} 的最小帧数为: {min_frame_count}")

        # 处理每个视频
        for source_path in paths:
            # 计算相对路径并构建目标路径
            relative_path = os.path.relpath(source_path, source_main_folder)
            target_path = os.path.join(target_main_folder, relative_path)

            # 确保目标子文件夹存在
            target_folder = os.path.dirname(target_path)
            ensure_dir_exists(target_folder)

            # 处理视频并保存
            process_and_save_video(source_path, target_path, min_frame_count, desired_fps)

    print("所有视频的帧率修改完成。")

if __name__ == "__main__":
    main()