import os
import glob
from moviepy.editor import VideoFileClip

def get_all_video_files(source_main_folder):
    """
    遍历主文件夹中的所有子文件夹，获取所有mp4文件的路径。
    返回一个字典，键为文件名，值为所有对应路径的列表。
    """
    video_dict = {}
    # 获取所有子文件夹
    subfolders = [f.path for f in os.scandir(source_main_folder) if f.is_dir()]
    
    for folder in subfolders:
        # 获取当前子文件夹中的所有mp4文件
        mp4_files = glob.glob(os.path.join(folder, '*.mp4'))
        for mp4 in mp4_files:
            filename = os.path.basename(mp4)
            if filename not in video_dict:
                video_dict[filename] = []
            video_dict[filename].append(mp4)
    
    return video_dict

def get_min_frame_count(video_paths):
    """
    给定一组视频路径，返回其中最小的帧数。
    """
    frame_counts = []
    for path in video_paths:
        try:
            with VideoFileClip(path) as clip:
                frame_count = int(clip.fps * clip.duration)
                frame_counts.append(frame_count)
        except Exception as e:
            print(f"无法读取视频 {path} 的帧数: {e}")
    return min(frame_counts) if frame_counts else None

def process_videos(video_dict, target_main_folder):
    """
    处理每组同名视频，将它们剪切到最小帧数，并保存到目标主文件夹中。
    """
    for filename, paths in video_dict.items():
        if len(paths) != 5:
            print(f"警告: 文件 {filename} 在 {len(paths)} 个文件夹中找到，而不是5个。跳过处理。")
            continue
        
        min_frame = get_min_frame_count(paths)
        if min_frame is None:
            print(f"无法确定文件 {filename} 的最小帧数。跳过处理。")
            continue
        
        # 计算最小持续时间
        durations = []
        for path in paths:
            try:
                with VideoFileClip(path) as clip:
                    durations.append(clip.duration)
            except Exception as e:
                print(f"无法读取视频 {path} 的持续时间: {e}")
        if not durations:
            print(f"无法获取文件 {filename} 的任何持续时间。跳过处理。")
            continue
        min_duration = min(durations)
        
        # 处理并保存每个视频
        for path in paths:
            try:
                with VideoFileClip(path) as clip:
                    # 剪切视频到最小持续时间
                    processed_clip = clip.subclip(0, min_duration)
                    
                    # 构建目标路径
                    relative_path = os.path.relpath(path, source_main_folder)
                    target_path = os.path.join(target_main_folder, relative_path)
                    
                    # 确保目标文件夹存在
                    os.makedirs(os.path.dirname(target_path), exist_ok=True)
                    
                    # 保存处理后的视频
                    processed_clip.write_videofile(target_path, codec='libx264', audio_codec='aac')
                    print(f"已处理并保存视频: {target_path}")
            except Exception as e:
                print(f"处理视频 {path} 时出错: {e}")

if __name__ == "__main__":
    # 定义源主文件夹和目标主文件夹路径
    source_main_folder = 'assets/VideoReconstruction'  # 替换为您的源主文件夹路径
    target_main_folder = 'assets/VideoReconstruction-processed'  # 替换为您的目标主文件夹路径
    
    # 获取所有视频文件
    video_dict = get_all_video_files(source_main_folder)
    
    # 处理视频
    process_videos(video_dict, target_main_folder)
    
    print("所有视频处理完成。")